#!/usr/bin/env ruby
# vim: set ts=2 sw=2 noet :

require 'socket'
#require '../lib/packet.rb'

class Amphibian
	# set up new amphibian structure
	# this uses fishnet's application port
	def initialize(host, port)  
		@realsock = TCPSocket.new(host, port)
		@host = host
		@port = port
	end

	# closes the connection
	# blocks until really closed
	def close
		@realsock.puts "close"
	        s = @realsock.gets
		@realsock.close
	        return true;
	end

	def neighbors
		@realsock.puts "neighbors"
	        s = @realsock.gets
	        raise "Amphibian Error" unless s =~ /neighbors (.*)/
	        return $1.split(/:/).map{|n|n.to_i}
	end
end

class AmphibianSender < Amphibian
	def initialize(host,port)
		super(host,port)
	end

	# Input:  a fish address to connect to
	#         a fish port on the given address
	# Output: true if connection is successful
	def connect(addr, port)
		@realsock.puts "connect #{addr} #{port}"
		s = @realsock.gets
		return false unless s =~ /connected/
		return true
	end

	# Input:  data to write to socket
	# Output: true if write is successful
	def write(data)
		@realsock.puts "write #{data.length}"
		@realsock.print data + "\n"
		s = @realsock.gets
		return false unless s =~ /wrote (\d+)/
		return $1
	end
end

class AmphibianAcceptor < Amphibian
	def initialize(host,port)
		super(host,port)
	end

	# Binds to the given port.
	# Input:  a fish port to listen on
	# Output: true if bind is successful
	def bind(port)
		@realsock.puts "bind #{port}"
		s = @realsock.gets
		return true if s =~ /bound/
		return false
	end

	# Blocks until a node connects to the bound port.
	# Output: a new AmphibianReceiver object
	#          -OR-
	#         nil on failure
	def accept
		@realsock.puts "accept"
		s = @realsock.gets
		return nil unless s =~ /(\d+)\s+(\d+)\s+(\d+)/
		return AmphibianReceiver.new(@host,@port,$1.to_i,$2.to_i,$3.to_i)
	end
end

class AmphibianReceiver < Amphibian
	def initialize(amphHost,amphPort,fishID,fishAddr,fishPort)
		super(amphHost,amphPort)
		@fishAddr, @fishPort = fishAddr, fishPort
		@realsock.puts "receiver #{fishID}"
		s = @realsock.gets
		@realsock = nil unless s =~ /receiver success/
	end
	attr_reader :fishAddr, :fishPort

	# Blocks until data is available.
	# Output: a String with (one..length) bytes of data
	#          -OR-
	#         nil if there is an error
	def read(length =0)
		return nil unless @realsock
		@realsock.puts "read #{length}"
	        case s = @realsock.gets
		when /closed/
		  return nil
		when /\d+/
		  length = $&.to_i
		end
		data = @realsock.read(length+1)
		return nil unless data and not data.empty?
		return data.chomp
	end
end
