class AmphibianMgr
	attr_accessor :node

	def initialize(sockets)
		@amphsock = nil
		@conns = {}
		@conn_from_id = {}
		@id_from_conn = {}
		@conn_from_port = {}
		@port_from_conn = {}
		@awaiting_read = {}

		@sockets = sockets
	end

	def start(addr, port)
		$stderr.puts "starting up amphibian on #{addr}:#{port}"
		@amphsock = TCPServer.new(addr, port)
		@sockets << @amphsock
	end

	def process(source)
		if source == @amphsock
			new_conn
			return true
		elsif @conns.has_key? source
			amph_try(source) { process_in(source) }
			return true
		end
		return false
	end

	def new_conn
		newconn = @amphsock.accept
		#puts "new conn " + newconn.inspect
		process_in(newconn)
		@sockets << newconn
		@conns[newconn] = true
	end

	def throw_away(conn)
		@sockets.delete(conn)
		@conns.delete(conn)
	end

	def amph_try(conn)
		begin
			yield
		rescue Errno::EPIPE
			puts "#{amphsock} is gone"
			throw_away(conn)
		end
	end

	def do_read(amphconn, len)
		id = @id_from_conn[amphconn]
		data = @node.transport_read(id, len)
		if data and data.length > 0
			amphconn.puts data.length
			amphconn.print data + "\n"
			@awaiting_read.delete(id)
		else
			@awaiting_read[id] = len
		end
	end

	def process_in(amphconn)
		cmdline = amphconn.gets
		unless cmdline and cmdline.length > 0
			throw_away(amphconn)
			return
		end
		#puts "amph cmd: #{cmdline}"
		cmd, args = cmdline.split(/ /, 2)
		args = args.split(/ /).map{|x| x.to_i} if args

		case cmd
		when /^connect/
			addr, port = args
		        id = @node.transport_socket
			@conn_from_id[id] = amphconn
			@id_from_conn[amphconn] = id
			@node.transport_connect(id, addr, port)
		when /^read/
			len = args[0]
			do_read(amphconn, len)
		when /^write/
			length = args[0]
			puts "req to write #{length} bytes of data"
			data = amphconn.read(length+1).chomp
			puts "req to write data #{data}"
			# read out a \r\n if there was one
			amphconn.read(1) if data[-1] == '\r'[0]
			id = @id_from_conn[amphconn]
			if n = @node.transport_write(id, data)
				amphconn.puts "wrote #{n}"
			else
			        amphconn.puts "write failed"
			end
	 	when /^bind/
	 		port = args[0]
		        id = @node.transport_socket
                        @id_from_conn[amphconn] = id
                        @conn_from_id[id] = amphconn
	 		if @node.transport_bind(id, port)
				@conn_from_port[port] = amphconn
				@port_from_conn[amphconn] = port
	 			amphconn.puts 'bound'
	 		else
	 			amphconn.puts 'bind failed'
	 		end
		when /^accept/                   
			id = @id_from_conn[amphconn]
			@node.transport_accept(id)
			# block!
		when /^receiver/
			id = args[0]
			@conn_from_id[id] = amphconn
			@id_from_conn[amphconn] = id
			amphconn.puts "receiver success"
		when /^close/
			id = @id_from_conn[amphconn]
			@node.transport_close(id)
		when /^neighbors/
			nbrs = @node.get_neighbors.map{|n|n.to_s}
			amphconn.puts "neighbors #{nbrs.join(':')}"
		else
			$stderr.puts "Amphibian: UNKNOWN COMMAND: #{cmd}"
		end
	end

	def transport_connected(id)
		raise TypeError if id.nil?
	  #$stderr.puts "transport_connected: id=#{id}"
	        amphconn = @conn_from_id[id]
		return ($stderr.puts "transport_connected: no connection on socket #{id}") unless amphconn
		amph_try(amphconn) {
			amphconn.puts "connected"
		}
	end

	def transport_accepted(id, new_id, remoteaddr, remoteport)
		raise TypeError if id.nil?
	  #$stderr.puts "transport_accepted: id=#{id}"
		amphconn = @conn_from_id[id]
		return ($stderr.puts "transport_accepted: no connection on socket #{id}") unless amphconn
		amph_try(amphconn) {
			amphconn.puts "#{new_id} #{remoteaddr} #{remoteport}"
		}
	end

	def transport_readready(id)
		raise TypeError if id.nil?
		if @awaiting_read[id]
			do_read(@conn_from_id[id], @awaiting_read[id])
		end
	end

	def transport_closed(id)
		raise TypeError if id.nil?
	        amphconn = @conn_from_id[id]
		return ($stderr.puts "transport_closed: no connection on socket #{id}") unless amphconn
	        amphconn.puts "closed"
	end
end
